#include "ai.h"
#include "manager.h"
#include <stdio.h>
#include <iostream>

const Component AngleTolerance = 5.0; // degrees
const Component TargetSpeed = 120.0; // px per s
const Component SpeedTolerance = 5.0; // px per s
const Component SpeedQuick = 40.0; // px per s

AiWaypoint::AiWaypoint(Manager *m) : GameObject(m) {
	radius = 30;
	number = 0;
	// Make the checkpoint invisible unless we're in edit mode.
	if (! manager->isEditMode()) {
		visible = false;
	}
};

void AiWaypoint::renderTo(const View &v)
{
	int red = makecol(255,0,0);
	Point p = v.translate(location);
	char text[200];
	sprintf(text, "WP %d", number);
	textout(v.bitmap, font, text, (int) p.x, (int) p.y + 10, red);
	circlefill(v.bitmap, (int) p.x, (int) p.y, 5, red);
}

void AiWaypoint::setLocation(const Vect3d &where)
{
	location = where;
	// Make sure we're on the ground.
	location.z = manager->getMap()->getHeight(Point(location.x, location.y));
}

AiController::AiController(Manager *m, HoverCraft *c) : manager(m), craft(c) {
	rudder = 0;
	power = 0;
	nextWaypoint = 0;
	printStuff = true;
}

void AiController::tick(Component quantum)
{
	AiWaypoint * nextwp = manager->getWaypoint(nextWaypoint);
	Vect3d relative = nextwp->getLocation() - craft->getLocation();
	Vect3d targetVel = relative;
	targetVel.normalise();
	targetVel = targetVel * TargetSpeed;
	Vect3d velDiff = targetVel - craft->getVelocity();
	Component targetAzimuth = toDegrees(atan2(-velDiff.y, velDiff.x));
	/*
	if (printStuff) {
		std::cout  << "Next WP:" << nextWaypoint << std::endl;
		std::cout  << "Target az.=" << targetAzimuth << std::endl;
		std::cout  << "Actual Azimuth:" << craft->getAzimuth() << std::endl;
		std::cout << "Velocity:" << craft->getVelocity().x << " " << craft->getVelocity().y << std::endl;
		std::cout << "Vel diff:" << velDiff.x << " " << velDiff.y << std::endl;
		printStuff =false;
	}
	*/
	Component azimuthDiff = targetAzimuth - craft->getAzimuth();
	if (azimuthDiff < -180) { azimuthDiff += 360.0; }
	if (azimuthDiff > 180) { azimuthDiff -= 360.0; }
	rudder = 0;
	if (azimuthDiff < -AngleTolerance) { rudder = -1; }
	if (azimuthDiff > AngleTolerance) { rudder = 1; }
	// Work out power...
	if ((velDiff.magnitudeSquared() > (SpeedTolerance * SpeedTolerance)) &&
		(fabs(azimuthDiff) < AngleTolerance))	 {
		power = 1;
	} else {
		power = 0;
	}
	// If we're really pointing the wrong way, turn the power on anyway.
	if (velDiff.magnitudeSquared() > (SpeedQuick *  SpeedQuick)) {
		power = 1;
	}
	//craft->setVelocity(targetVel);
}

void AiController::notifyCollision(GameObject *obj)
{
	AiWaypoint * wp = dynamic_cast<AiWaypoint *>(obj);
	if (wp) {
		if (wp->getNumber() == nextWaypoint) {
			nextWaypoint++;
			if (nextWaypoint >= manager->getNumWaypoints()) {
				nextWaypoint = 0;
			}
			printStuff = true;
		}
	}
}

