// gameobject.cpp
#include "gameobject.h"
#include "manager.h"
#include "data.h"
#include "ai.h"

#include <allegro.h>
#include <iostream>
#include <stdio.h>
#include <math.h>

using namespace std;

int StaticObject::spriteRefs[] = { FLAG_TGA, SMALLTREE1_TGA, SMALLTREE2_TGA, TREE1_TGA, TREE2_TGA};

StaticObject::StaticObject(Manager *m, StaticObjectType t) : GameObject(m), type(t)
{
	setSprite();
}

void StaticObject::setSprite()
{
	if ((int) type > (int) Tree2) {
		// SOmething's afoot...
		type = Flag;
	}
	spritebm = manager->getSprite(spriteRefs[(int) type]);
	radius = (spritebm->w) / 2;
	solid = true;
}

void StaticObject::setLocation(const Vect3d &where)
{
	location = where;
	// Make sure we're on the ground.
	location.z = manager->getMap()->getHeight(Point(location.x, location.y));
}

void StaticObject::renderTo(const View &v)
{
	Point p = v.translate(location);
	// cout << "Point translated " << p.x << "," << p.y << endl;
	// Make it aligned with the centre base...
	p.x -= ((spritebm->w) / 2);
	// Assume that the "hot spot" is 1/5 of the way up.
	p.y -= ((Component) (spritebm->h)) * 0.8;
	// set_alpha_blender();
	draw_sprite(v.bitmap, spritebm, (int) p.x, (int) p.y);
}

// EditorCursor

EditorCursor::EditorCursor(Manager *m) : GameObject(m) {
	radius = 5;
	reloadTime = 0;
}

void EditorCursor::renderTo(const View &v)
{
	// Draw a small white circle
	Point p = v.translate(location);
	p.y -= radius;
	int white = makecol(255,255,255);
	circlefill(v.bitmap, (int) p.x, (int) p.y, (int) radius, white);
}

void EditorCursor::plonkItem(GameObject *item)
{
	item->setLocation(location);
	manager->addObject(item);
	reloadTime = 1.0;
}

void EditorCursor::deleteItem()
{
	GameObject * obj = manager->findNearest(this);
	if (obj != 0) {
		obj->setDead(true);
	} else {
		std::cout << "Nothing to delete " << std::endl;	
	}
	reloadTime = 1.0;
}

void EditorCursor::snapItem()
{
	GameObject * obj = manager->findNearest(this);
	if (obj != 0) {
		obj->setLocation(getLocation());
	} else {
		std::cout << "Nothing to snap" << std::endl;	
	}
	reloadTime = 1.0;
}

void EditorCursor::tick(Component quantum)
{
	const Component Speed = 400.0;
	Component xdir = 0;
	Component ydir = 0;
	if (key[KEY_A]) { ydir = Speed; }
	if (key[KEY_X]) { ydir = -Speed; }
	if (key[KEY_S]) { xdir = Speed; }
	if (key[KEY_Z]) { xdir = -Speed; }
	
	setLocation(Vect3d(location.x + (xdir * quantum), location.y + (ydir * quantum), 0));
	if (reloadTime > 0) {
		reloadTime -= quantum;
	} else {
		if (key[KEY_1]) { plonkItem(new StaticObject(manager,StaticObject::SmallTree1)); }
		if (key[KEY_2]) { plonkItem(new StaticObject(manager,StaticObject::SmallTree2)); }
		if (key[KEY_3]) { plonkItem(new StaticObject(manager,StaticObject::Tree1)); }
		if (key[KEY_4]) { plonkItem(new StaticObject(manager,StaticObject::Tree2)); }
		if (key[KEY_5]) { plonkItem(new StaticObject(manager,StaticObject::Flag)); }
		if (key[KEY_C]) { plonkItem(new CheckPoint(manager)); }
		if (key[KEY_W]) { plonkItem(new AiWaypoint(manager)); }
		if (key[KEY_F1]) { plonkItem(new Powerup(manager,Powerup::Freeze)); }
		if (key[KEY_F2]) { plonkItem(new Powerup(manager,Powerup::OutOfControl)); }
		if (key[KEY_F3]) { plonkItem(new Powerup(manager,Powerup::FreezeOpponents)); }
		if (key[KEY_F4]) { plonkItem(new Powerup(manager,Powerup::Turbo)); }
		
		if (key[KEY_D]) { deleteItem(); }
		if (key[KEY_HOME]) { snapItem(); }
	}
	Component cpmod = 0;
	if (key[KEY_EQUALS]) cpmod += 8;
	if (key[KEY_MINUS]) cpmod -= 8;
	if (cpmod) {
		GameObject * obj = manager->findNearest(this);
		if (obj) {
			CheckPoint * cp = dynamic_cast<CheckPoint *>(obj);
			if (cp) {
				if (key[KEY_LSHIFT]) {
					cp->setAzimuth(cp->getAzimuth() + cpmod);
				} else {
					cp->setRadius(cp->getRadius() + cpmod);
				}
			}
		}
	}
}

void EditorCursor::setLocation(const Vect3d &where)
{
	location = where;
	// Make sure we're on the ground.
	location.z = manager->getMap()->getHeight(Point(location.x, location.y));
}

CheckPoint::CheckPoint(Manager *m) : GameObject(m) {
	radius = 40;
	number = 0;
	// Make the checkpoint invisible unless we're in edit mode.
	if (! manager->isEditMode()) {
		visible = false;
	}
	azimuth = 0;
}

void CheckPoint::drawSpot(const View &v, Vect3d where)
{
	where.z = manager->getMap()->getHeight(Point(where.x, where.y));
	// Draw a small yellow circle
	Point p = v.translate(where);
	int yellow = makecol(255,255,0);
	circlefill(v.bitmap, (int) p.x, (int) p.y, 4, yellow);
}

void CheckPoint::renderTo(const View &v)
{
	int white = makecol(255,255,255);
	for (Component c = 0; c < (M_PI * 2); c += (M_PI / 6)) {
		Vect3d diff(sin(c) * radius, cos(c) * radius, 0);
		drawSpot(v, location + diff);
	}
	Point p = v.translate(location);
	char text[200];
	sprintf(text, "CP %d", number);
	textout(v.bitmap, font, text, (int) p.x, (int) p.y, white);
	Point p1 = v.translate(location - right);
	Point p2 = v.translate(location + right);
	line(v.bitmap, (int) p1.x, (int) p1.y, (int) p2.x, (int) p2.y, white);
	p1 = v.translate(location);
	p2 = v.translate(location + (forward * radius));
	line(v.bitmap, (int) p1.x, (int) p1.y, (int) p2.x, (int) p2.y, white);
}

void CheckPoint::setLocation(const Vect3d &where) {
	location = where;
}

void CheckPoint::setVectors()
{
	if (azimuth > 360.0) azimuth -= 360.0;
	if (azimuth < 0.0) azimuth += 360.0;
	forward = Vect3d(cos(toRadians(azimuth)), - sin(toRadians(azimuth)), 0);
	// Manual hack: rotate by 90 deg
	right.y = -forward.x;
	right.x = forward.y;
	right.z = 0;
	right = right * radius;
}

void CheckPoint::load(std::istream &in)
{
	in >> radius >> number >> azimuth; 
	setVectors(); 
	if ((number +1) > manager->getNumCheckpoints()) {
		manager->setNumCheckpoints(number + 1);
	}
}

// Powerup methods
Powerup::Powerup(Manager *m, PowerupType t) : GameObject(m)
{
	type = t;
	animationTime = 0;
	radius = 16;
	respawnTime = 0;
	visible = true;
}

void Powerup::renderTo(const View &v)
{
	int sprite;
	switch (type) {
		case Freeze:
			sprite = ICECUBE_TGA;
			break;
		case OutOfControl:
			sprite = CRAZY_TGA;
			break;
		case FreezeOpponents:
			sprite = ICECUBES_TGA;
			break;
		case Turbo:
			sprite = TURBO_TGA;
			break;
		default:
			sprite = TURBO_TGA;
	}
	BITMAP *spritebm = manager->getSprite(sprite);
	Point p = v.translate(location);
	p.x -= ((spritebm->w) / 2);
	// Assume that the "hot spot" is 1/5 of the way up.
	p.y -= ((Component) (spritebm->h)) * 0.8;
	// Mak eit bounce
	p.y -= fabs(animationTime - 0.5) * 16;
	draw_sprite(v.bitmap, spritebm, (int) p.x, (int) p.y);
}

void Powerup::tick(Component quantum)
{
	if (respawnTime > 0 ) {
		respawnTime -= quantum;
		if (respawnTime < 0) {
			visible = true;
			collidable = true;
		}
	}
	animationTime += quantum;
	if (animationTime > 1) {
		animationTime -= 1;
	}
}

void Powerup::handlePlayerCollision(HoverCraft *h)
{
	visible = false;
	respawnTime = 35.0;
	collidable = false;
	// make it have an effect.
	switch (type) {
		case Freeze:
			h->freeze();
			break;
		case OutOfControl:
			h->wonky();
			break;
		case FreezeOpponents:
			manager->freezeOpponents(h);
			break;
		case Turbo:
			h->addTurbo();
			break;
	}
}
