#include "map.h"
#include "constants.h"
#include "manager.h"
#include "ui.h"
#include "language.h"

#include <iostream>
#include <allegro.h>

using namespace std;

void ShowError(const char *msg)
{
	cerr << msg << endl;
}

// Video mode selection... blanatly stolen from my tins2003 entry
// "Jelly attack"
static int depths[] = { 16,32,24,0 };
static bool SetupMode(bool fullscreen)
{
	int chosenDepth = 0;
	int gfx_mode = GFX_AUTODETECT_WINDOWED;
	if (fullscreen) gfx_mode = GFX_AUTODETECT_FULLSCREEN;
	// Try several colour depths
	for (int i=0; depths[i] != 0; i++) {
		set_color_depth(depths[i]);
		int res =
			set_gfx_mode(gfx_mode,ScreenWidth,ScreenHeight,ScreenWidth,ScreenHeight);
		if (res == 0)
		{
			chosenDepth = depths[i];
			return true;
		}
	}
	ShowError("Cannot set gfx mode");
	return false;
}

static void runGame(int level)
{
	Manager manager(false);
	manager.preview(level);
	manager.initLevel(level);
	manager.mainLoop();
}

static void runEditor(int level)
{
	language::current = & language::English;	
	Manager manager(true);
	manager.musicOn = false;
	manager.preview(level);
	manager.initLevel(level);
	manager.mainLoop();
}

static void doMenus()
{
	uiInit();
	if (! uiSelectLanguage()) {
		return;
	}
	int level = 0;
	while ((level = uiMainMenu())) 
	{
		runGame(level);
	}
}

int main(int argc, char *argv[])
{
	allegro_init();
	bool fullscreen = false;
	bool edit = false;
	int level = 1;
	for (int i=1; i<argc; i++) {
		if (argv[i][0] == 'f') {
			fullscreen = true;
		}
		if (argv[i][0] == 'e') {
			edit = true;
		}
		if (atoi(argv[i]) != 0) {
			level = atoi(argv[i]);
		}
	}
	SetupMode(fullscreen);
	install_mouse();
	install_keyboard();
	install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, "");
	if (edit) {
		runEditor(level);
	} else {
		doMenus();
	}
	return 0;
}
END_OF_MAIN()

