// map.h

#ifndef MAP_INCLUDED
 #define MAP_INCLUDED

#include <allegro.h>
#include "vector3d.h"

class Point {
	public:
		Component x;
		Component y;
		Point(Component a, Component b) : x(a),y(b) { }
		Point() : x(0),y(0) { }
};

class View {
	public:
		Point bottomLeft; // screen target coordinates of (x=0 y=0 z=0)
		BITMAP *bitmap;
		View(const Point &bl, BITMAP *bm) : bottomLeft(bl), bitmap(bm) { }
		View() { }
		Point translate(const Vect3d &v) const;
};

class Map {
	private:
		BITMAP *heightmap; // Bitmap containing colours for heights
		BITMAP *colourmap; // Bitmap containing colours for heights
		DATAFILE *df;
		Component **heightarray; // Actual heights
		Vect3d **normalarray; // Surface normals.
	public:
		Map(); // default constructor
		static const int BlocksWide = 256;
		static const int BlockSize = 12; // pixels
		Component getHeight(const Point & p);
		Vect3d getNormal(const Point &p);
		bool isWater(const Point &p);
		void load(int level); // TODO: specify which level to load
		void renderTo(const View &v);
		~Map();
	private:
		Vect3d getPoint(int i, int j);
		int getColour(int i, int j);
		void calculateNormals();
		void destroyBitmaps();
};

#endif // MAP_INCLUDED
